import { Request, Response, RequestHandler } from 'express';
import { FakeClient } from './server-rendering';
import SessionStore from './sessions/SessionStore';
import BicycleRequest from './types/Request';
import Logging from './types/Logging';
import MutationResult from './types/MutationResult';
import Query from './types/Query';
import Schema from './types/Schema';
import ServerPreparation from './types/ServerPreparation';
import ServerResponse from './types/ServerResponse';
import { BaseRootQuery, Mutation } from './typed-helpers/query';
import { Ctx } from './Ctx';
import NetworkLayerInterface from './types/NetworkLayerInterface';
/**
 * Options for BicycleServer
 *
 * @public
 */
export interface Options {
    /**
     * silence error logging
     *
     * @default false
     */
    readonly disableDefaultLogging?: boolean;
    /**
     * a custom session store
     *
     * @default MemorySessionStore
     */
    readonly sessionStore?: SessionStore;
    /**
     * The maximum number of active sessions.
     *
     * You can also set via the `BICYCLE_SESSION_STORE_SIZE` environment
     * variable.
     *
     * @defualt 100
     */
    readonly sessionStoreSize?: number;
    /**
     * Event triggered when an error is thrown by
     * a mutaiton or query resolver
     *
     * @default no-op
     */
    readonly onError?: (e: {
        error: Error;
    }) => any;
    /**
     * Event triggered when a request starts
     *
     * @default no-op
     */
    readonly onRequestStart?: (e: {
        readonly request: BicycleRequest;
    }) => any;
    /**
     * Event triggered when a request ends
     *
     * @default no-op
     */
    readonly onRequestEnd?: (e: {
        readonly request: BicycleRequest;
        readonly response: ServerResponse;
    }) => any;
    /**
     * Event triggered when a mutation starts
     *
     * @default no-op
     */
    readonly onMutationStart?: (e: {
        mutation: {
            readonly method: string;
            readonly args: Object;
        };
    }) => any;
    /**
     * Event triggered when a mutation ends
     *
     * @default no-op
     */
    readonly onMutationEnd?: (e: {
        mutation: {
            readonly method: string;
            readonly args: Object;
        };
        result: MutationResult<any>;
    }) => any;
    /**
     * Event triggered when a query starts
     *
     * @default no-op
     */
    readonly onQueryStart?: (e: {
        query: Object;
    }) => any;
    /**
     * Event triggered when a query ends
     *
     * @default no-op
     */
    readonly onQueryEnd?: (e: {
        query: Object;
        cacheResult: Object;
    }) => any;
}
export default class BicycleServer<Context> {
    _logging: Logging;
    _schema: Schema<Context>;
    _sessionStore: SessionStore;
    constructor(schema: Schema<Context>, options?: Options);
    runQuery<TResult>(query: BaseRootQuery<TResult>, context: Ctx<Context>): Promise<TResult>;
    runQuery(query: Query, context: Ctx<Context>): Promise<any>;
    runMutation<TResult>(mutation: Mutation<TResult>, context: Ctx<Context>): Promise<TResult>;
    runMutation(method: string, args: any, context: Ctx<Context>): Promise<any>;
    handleMessage(message: BicycleRequest, getContext: (options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>): Promise<ServerResponse>;
    createMiddleware(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>): RequestHandler;
    createServerRenderer<TResult>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response) => TResult): (req: Request, res: Response) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, a1: TArg1) => TResult): (req: Request, a1: TArg1) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2, TArg3>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    createServerRenderer<TResult, TArg1, TArg2, TArg3, TArg4>(getContext: (req: Request, res: Response, options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>, fn: (client: FakeClient, req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3, a4: TArg4) => TResult): (req: Request, res: Response, a1: TArg1, a2: TArg2, a3: TArg3, a4: TArg4) => Promise<{
        serverPreparation: ServerPreparation;
        result: TResult;
    }>;
    getNetworkLayer(getContext: (options: {
        stage: 'query' | 'mutation';
    }) => Ctx<Context>): NetworkLayerInterface;
    getActiveSessionCount(): number | null;
    getMaxSessionCount(): number | null;
}
